/************************************************************/
/*                                                          */
/* THIS IS A DEMO PROGRAM FOR USING THE HUNT ENGINEERING    */
/* HERON_API TO ACCESS HERON FIFOS                          */
/*                                                          */
/*                                                          */
/*                                                          */
/*     PW    ver 1.0    22-7-03    first written            */
/************************************************************/



#define INFIFONO  0       /* the FIFO number you choose for input */
#define OUTFIFONO  1      /* the FIFO number you choose for output */

#include <stdio.h>
#include <std.h>

/* Use HERON-API so you need the next include line*/
/* it must be set for the module type you use -   */
/* the create new project plug in will edit it for you */
#include "heron4.h"


/* defines used specifically in this program*/
#define BUFFER_SIZE     256   /* number of words to be used */ 
    
HeronFIFO *infifo;
HeronFIFO *outfifo;                           
       
#pragma FUNC_EXT_CALLED(maintask)
        
main()
{
}        
        
       
maintask (void)
{
/* WARNING the default stack size for this task ( maintask) defined in    */
/* the cdb file is 8192 bytes                                             */
/* This means that you cannot statically declare more than 2048 integers  */
/* in this task.                                                          */
/* You can increase the stack size allowed by changing the settings for   */
/* the task in the cdb file.                                              */
/* If you need more than 2048 integers, it is better to use Malloc/Free   */
/* to allocate your data buffers 					  */

int rstatus,wstatus;                      
int i,count;
int *pattern;
int *results;
          
printf("Hello from HERON-API fifo demo.\n");

/* open the FIFO for reading*/
infifo=HeronOpenFifo(INFIFONO,"r");
if (infifo==NULL)
     {
       printf("error opening FIFO %d, error is %x\n",INFIFONO,heronerr);
     }
     
/* open the FIFO for reading*/
outfifo=HeronOpenFifo(OUTFIFONO,"w");
if (outfifo==NULL)
     {
       printf("error opening FIFO %d, error is %x\n",OUTFIFONO,heronerr);
     }

/* allocate the data buffers (in external memory) */
pattern=(int *)malloc(BUFFER_SIZE*sizeof(int));
results=(int *)malloc(BUFFER_SIZE*sizeof(int));
if ((results==NULL) || (pattern==NULL))
     {
       printf("error with allocating the data buffers\n");
     }


/* RELEASE THE CONFIG LINE */
HeronConfigOff(); 

count=0;
while(1)
{    
/* initialise the buffers */
for (i=0;i<BUFFER_SIZE;i++)
{
	pattern[i]=i;
	results[i]=0;
}    
        
/* start sending the buffer*/
wstatus = HeronWrite(outfifo,pattern,BUFFER_SIZE); 
if (wstatus != HERON_IO_IN_PROGRESS)
	{       
    printf("error from write\n");
	exit(0);	
	}
	
/*wait for the write transfer to complete */         
wstatus = HeronWaitIo(outfifo);    
if (wstatus != HERON_OK)
	{       
    printf("error from waitio\n");
	exit(0);	
	}
	                       
/* now start to read the data back from the FIFO */
rstatus = HeronRead(infifo,results,BUFFER_SIZE); 
if (rstatus != HERON_IO_IN_PROGRESS)
	{       
    printf("error from read\n");
	exit(0);	
	}	
	
/*wait for the read transfer to complete */         
rstatus = HeronWaitIo(infifo);    
if (rstatus != HERON_OK)
	{       
    printf("error from waitio\n");
	exit(0);	
	} 	
	                                         
/* check the data */
for (i=0;i<BUFFER_SIZE;i++)
{
if (results[i]!=i) printf("error received %x not %x at position %x\n",results[i],i,i);
}         
	  
printf("completed loop %d\n",count++);    
	      
}

/* loops are done, so tidy up - actually we won't reach here because of while(1)*/
rstatus = HeronClose(infifo);
	if (rstatus != HERON_OK)
	{
	printf(" Error closing read fifo \n");
	exit(0);
	}
wstatus = HeronClose(outfifo);
	if (wstatus != HERON_OK)
	{
	printf(" Error closing write fifo \n");
	exit(0);
	}

} /* end maintask */
